var gameFlag = false;
var score = 0;
var timerObj = null;
var gameObj = null;

var bgImage = null;
var hitImage = null;
var itemImages =null;

// 初期化処理
function initial(){
	bgImage = new Image();
	bgImage.src = 'images/background.png';
	bgImage.onload = function(){
		drawBackground();
	}
	hitImage = new Image();
	hitImage.src = 'images/hit.png';
	itemImages = [new Image(), new Image(), new Image()];
	itemImages[0].src = 'images/item1.png';
	itemImages[1].src = 'images/item2.png';
	itemImages[2].src = 'images/item3.png';
}

// スタート
function start(){
	if (gameFlag == false){
		score = 0;
		gameFlag = true;
		gameObj = new GameObject();
		timerObj = new TimerObject();
	}
}

// マウス操作
function move(event){
	if (gameFlag){
		gameObj.move(event);
	}
}

// 背景描画
function drawBackground(){
	var context = canvas.getContext('2d');
	context.drawImage(bgImage, 0, 0);
	if (gameFlag == false){
		context.font="50pt Georgia";
		context.fillStyle = 'red';
		context.fillText("Collect It!",250,200);
		context.font="30pt Georgia";
		context.fillText("click to start",300,400);
	}
}
	
// タイマー処理
function TimerObject(){
	var counter = 0;
	var level = 50;
	
	this.runNow = function(){
		counter += 1;
		if (counter > level){
			counter = 0;
			if (level > 2){
				level--;
			}
			gameObj.addItem();
		}
		gameObj.run();
	}
	this.stop = function(){
		clearInterval(timer);
		gameFlag = false;
	}
	var timer = setInterval(this.runNow, 50);
}

// ゲーム管理
function GameObject(){
	var missCount = 0;
	var me = this;
	var canvas = document.querySelector('#canvas');
	var myChar = new CharacterObject();;
	var items = [];
	var mouseX = 0;
	var mouseY = 0;
		
	// アイテム追加
	this.addItem = function(){
		var n = Math.floor( Math.random() * 3);
		items.push(new GameItem(itemImages[n]));
	}
	
	// マウス
	this.move = function(e){
		mouseX = e.clientX - canvas.offsetLeft;
		mouseY = e.clientY - canvas.offsetTop;
	}

	// スコア表示
	this.drawScore = function(){
		var context = canvas.getContext('2d');
		context.font="48px Georgia";
		context.fillStyle = 'red';
		context.fillText(score,20, 50);
	}
	
	// タイマー処理
	this.run = function(){
		myChar.move(mouseX, mouseY);
		this.checkHit();
		if (missCount >= 10){
			timerObj.stop();
		}
		drawBackground();
		for (var n in items){
			if (items[n].draw(canvas) == false){
				if (items[n].isHit() == false){
					missCount++;
				}
				items.splice(n,1);
			}
		}
		myChar.draw(canvas);
		this.drawScore();

	}
	
	// ヒットチェック
	this.checkHit = function(){
		var point = myChar.point();
		for (var n in items){
			var point2 = items[n].point();
			var dx = Math.abs(Math.abs(point[0]) - Math.abs(point2[0]));
			var dy = Math.abs(Math.abs(point[1]) - Math.abs(point2[1]));
			var d = Math.sqrt(dx * dx + dy * dy);
			if (d < 75){
				items[n].hit();
			}
		}
	}
}

// キャラクタ
function CharacterObject(){
	var x = 350;
	var y = 250;
	var image = new Image();
		image.src = 'images/character.png';
	
	// 動かす
	this.move = function(mx, my){
		if (mx > x + 50){ x += 5; }
		if (mx < x + 50){ x-= 5; }
		if (my > y + 50){ y+= 5; }
		if (my < y + 50){ y-= 5; }
	}
	
	// 描画
	this.draw = function(canvas){
		var context = canvas.getContext('2d');
		context.drawImage(image, x, y);
	}
	
	// 位置を返す
	this.point = function(){
		return [x, y];
	}
}

// アイテム
function GameItem(img){
	var hitFlag = false;
	var image = img;
	var lastCount = Math.floor( Math.random() * 100) + 50;
	var count = 0;
	var x = Math.floor( Math.random() * 700);
	var y = Math.floor( Math.random() * 500);
	
	// 描画する
	this.draw = function(canvas){
		count++;
		var context = canvas.getContext('2d');
		context.globalAlpha = count / lastCount;
		context.drawImage(image, x, y);
		context.globalAlpha = 1.0;
		return count < lastCount;
	}
	
	// 位置を返す
	this.point = function(){
		if (hitFlag){
			return [-1000, -1000];
		} else {
			return [x, y];
		}
	}
	
	// ヒット処理
	this.hit = function(){
		score+= (lastCount - count) * 10 + 100;
		image = hitImage;
		hitFlag = true;
		setTimeout(this.clear,200);
	}
	
	// ヒットフラグ
	this.isHit = function(){
		return hitFlag;
	}
		
	// 消去
	this.clear = function(){
		count = lastCount;
	}
}
